import json
import logging
import random
import uuid
import xml.etree.ElementTree as ET
from functools import lru_cache
from io import BytesIO

import schedulesy_qrcode.generate
from schedulesy_qrcode.generate import multi
from schedulesy_qrcode.render import render

logger = logging.getLogger(__name__)


class ADE_Parser:
    FLAT = "flat.json"
    tree = {}

    def __init__(self, client):
        self.s3_client = client
        self.flat = (
            {"rooms": {}, "paths": {}}
            if not self.s3_client.exists(ADE_Parser.FLAT)
            else json.loads(self.s3_client.get(ADE_Parser.FLAT))
        )

    def hash_room(self, room):
        content = room["path"].copy()
        content.append(room["name"])
        content.append(",".join(list(map(str, room["color"]))))
        return self.hash_path(content)

    def hash_path(self, path):
        return self._hash("%%".join(path))

    @lru_cache(maxsize=1000)
    def _hash(self, path):
        return str(uuid.uuid3(uuid.NAMESPACE_DNS, path))

    def random_color(self):
        color = [random.randint(0, 255) for _ in range(3)]
        color[random.randint(0, 2)] = 0
        return color

    def dig(self, path, element, center_color, branch):
        rooms = []

        def compare(room_list, building_color):
            final_color = building_color
            if len(room_list) == 4:
                changed = False
                branch[f"Planche {multi(rooms)}"] = {}
                for room in room_list:
                    if self.hash_path(room["path"]) in self.flat["paths"]:
                        final_color = self.flat["paths"][self.hash_path(room["path"])]
                        room["color"] = final_color
                    if room["id"] in self.flat["rooms"]:
                        changed |= room["id"] not in self.flat["rooms"] or self.flat[
                            "rooms"
                        ][room["id"]] != self.hash_room(room)
                    else:
                        changed = True
                    branch[f"Planche {multi(rooms)}"][room['name']] = room
                self.s3_client.clean(rooms)
                if not changed:
                    changed |= not self.s3_client.exists(f'{multi(rooms)}.png')
                if changed:
                    for room in room_list:
                        self.flat["rooms"][room["id"]] = self.hash_room(room)
                        self.flat["paths"][self.hash_path(room["path"])] = final_color
                    schedulesy_qrcode.generate.generate(
                        room_list, final_color, self.s3_client
                    )
                    self.save_progress()
                room_list = []
            return room_list

        for child in element:
            if child.tag == "leaf":
                rooms.append(
                    {
                        "id": child.attrib["id"],
                        "path": path,
                        "name": child.attrib["name"],
                        "color": center_color,
                    }
                )
                rooms = compare(rooms, center_color)
            if child.tag == "branch":
                branch[child.attrib["name"]] = {}
                sub_d = branch[child.attrib["name"]]
                new_path = path.copy()
                new_path.append(child.attrib["name"])
                color = self.random_color()
                self.dig(new_path, child, color, sub_d)
        if len(rooms) > 0:
            # logger.info("Duplicating rooms")
            while len(rooms) < 4:
                rooms.append(rooms[0])
            rooms = compare(rooms, center_color)

    def save_progress(self):
        self.s3_client.upload(
            BytesIO(json.dumps(self.tree).encode("UTF-8")),
            "tree.json",
            "application/json",
        )
        self.s3_client.upload(
            BytesIO(json.dumps(self.flat).encode("UTF-8")),
            ADE_Parser.FLAT,
            "application/json",
        )
        # open("out.json", "w").write(json.dumps(self.tree))
        # open("flat.json", "w").write(json.dumps(self.flat))

    def parse(self, content):
        try:
            classrooms = ET.fromstring(content)[0]
        except Exception as ex:
            logger.error(content)
            logger.error(str(ex))
            raise (ex)
        self.dig([], classrooms, self.random_color(), self.tree)

        self.save_progress()
        ordered_tree = json.loads(json.dumps(self.tree, sort_keys=True))
        # open("ordered_tree.json", "w").write(json.dumps(ordered_tree))
        self.s3_client.upload(
            BytesIO(render(ordered_tree).encode("UTF-8")), "index.html", "text/html"
        )
        open("index.html", "w").write(render(ordered_tree))



import functools
import io
import logging
import re

import qrcode
from PIL import Image, ImageDraw, ImageFont, ImageOps
from qrcode.image.styledpil import StyledPilImage
from qrcode.image.styles.colormasks import RadialGradiantColorMask
from qrcode.image.styles.moduledrawers import RoundedModuleDrawer

from schedulesy_qrcode.config import FONT_CONF, QR_CONF

logger = logging.getLogger(__name__)


def generate(rooms, color, client):
    def save_image(image, filename):
        output = io.BytesIO()
        image.save(output, "png")
        output.seek(0)
        client.upload(output, filename, "image/png")
        output.close()

    def single_room(room):
        @functools.cache
        def normalize(input):
            return re.sub(r"^([A-Z]{3}) *-([A-Z])", "\\1 - \\2", input, 0)

        if 'path' in room and len(room["path"]):
            room["path"][0] = normalize(room["path"][0])

        logger.info(
            f'🎨 Generating {".".join(room["path"])}.{room["name"]} ({room["id"]})'
        )
        qr = qrcode.QRCode(error_correction=qrcode.constants.ERROR_CORRECT_H)

        qr.add_data(f'{QR_CONF["url"]}/public/{room["id"]}')

        image = qr.make_image(
            image_factory=StyledPilImage,
            module_drawer=RoundedModuleDrawer(),
            color_mask=RadialGradiantColorMask(
                back_color=(255, 255, 255), center_color=color, edge_color=(0, 0, 0)
            ),
            embeded_image_path=QR_CONF['logo'],
        )

        header = QR_CONF['header']

        def split(a, n):
            k, m = divmod(len(a), n)
            return (
                a[i * k + min(i, m) : (i + 1) * k + min(i + 1, m)] for i in range(n)
            )

        footer = "\n".join(
            [" - ".join(x) for x in list(split(room["path"] + [room["name"]], 3))]
        )

        expanded_image = ImageOps.expand(image, border=20, fill="white")

        # Add define a new font to write in the border
        big_font = ImageFont.truetype(FONT_CONF['path'], int(FONT_CONF['header']))
        small_font = ImageFont.truetype(FONT_CONF['path'], int(FONT_CONF['footer']))

        # Instantiate draw object & add desired text
        draw_object = ImageDraw.Draw(expanded_image)
        draw_object.text(xy=(60, 10), text=header, fill=(0, 0, 0), font=big_font)

        draw_object.text(
            xy=(60, expanded_image.height - 55),
            text=footer,
            fill=(0, 0, 0),
            font=small_font,
        )

        bordered = ImageOps.expand(expanded_image, border=10, fill=tuple(color))

        # Preview the image
        # bordered.show()

        # Save the image
        # bordered.save(f'out/{room["id"]}.png')
        save_image(bordered, f'{room["id"]}.png')
        return bordered

    images = list(map(single_room, rooms))

    w, h = images[0].size

    separation = 2

    # create big empty image with place for images
    new_image = Image.new(
        "RGB", (w * 2 + separation, h * 2 + separation), color="white"
    )

    # put images on new_image
    new_image.paste(images[0], (0, 0))
    new_image.paste(images[1], (w + separation, 0))
    new_image.paste(images[2], (0, h + separation))
    new_image.paste(images[3], (w + separation, h + separation))

    s_ids = multi(rooms)
    # save it
    logger.info(f"🎨 Generating {s_ids}")
    save_image(new_image, f"{s_ids}.png")


def multi(rooms):
    ids = [int(room["id"]) for room in rooms]
    ids.sort()
    return "-".join(list(map(str, ids)))



import io
import logging

import boto3
from botocore.exceptions import ClientError

from schedulesy_qrcode.config import S3_CONF
from schedulesy_qrcode.generate import multi

logger = logging.getLogger(__name__)


class S3_client:
    def __init__(self):
        self.client = boto3.client(
            "s3",
            aws_access_key_id=S3_CONF['access_key'],
            aws_secret_access_key=S3_CONF['secret_key'],
            endpoint_url=S3_CONF['endpoint'],
        )

        self.bucket = S3_CONF['bucket']

        response = self.client.list_buckets()

        if self.bucket in [b["Name"] for b in response["Buckets"]]:
            logger.info(f"🪣 Bucket {self.bucket} already exists")
        else:
            logger.info(f"🪣 Creating bucket {self.bucket}")
            self.client.create_bucket(Bucket=self.bucket)
            self.client.put_bucket_cors(
                Bucket=self.bucket,
                CORSConfiguration={
                    "CORSRules": [
                        {
                            "AllowedMethods": ["GET", "HEAD"],
                            "AllowedOrigins": [
                                "*",
                            ],
                            "ExposeHeaders": ["*"],
                            "AllowedHeaders": ["Content-Type", "Authorization"],
                        }
                    ]
                },
            )

    def upload(self, content, filename, mime_type):
        logger.info(f"⬆️ Uploading file {filename}")
        self.client.upload_fileobj(
            content,
            self.bucket,
            filename,
            ExtraArgs={
                "ContentType": mime_type,
                "ACL": "public-read",
            },
        )

    def get(self, filename):
        logger.info(f"⬇️ Downloading {filename}")
        output = io.BytesIO()
        self.client.download_fileobj(self.bucket, filename, output)
        return output.getvalue()

    def exists(self, filename):
        try:
            self.client.get_object_acl(Bucket=self.bucket, Key=filename)
        except ClientError as ex:
            return ex.response['Error']['Code'] != 'NoSuchKey'
        return True

    def clean(self, rooms):
        room_ids = [int(room["id"]) for room in rooms]
        for prefix in room_ids:
            response = self.client.list_objects(Bucket=self.bucket, Prefix=f'{prefix}-')
            # logger.info(f'{prefix}- {response}')
            if response.get('Contents') and len(response.get('Contents')) > 0:
                for filename in [
                    i['Key']
                    for i in response.get('Contents')
                    if i['Key'] != f'{multi(rooms)}.png'
                ]:
                    logger.info(f"🗑️ Removing {filename}")
                    self.client.delete_object(Bucket=self.bucket, Key=filename)



import logging
import xml.etree.ElementTree as ET

import requests

from schedulesy_qrcode.ceph import S3_client
from schedulesy_qrcode.config import ADE_CONF
from schedulesy_qrcode.parse import ADE_Parser

BASE_URL = ADE_CONF['url']

logger = logging.getLogger(__name__)


def refresh():
    logger.info("🔗 Connecting to ADE")
    response = requests.get(
        BASE_URL,
        params={
            "login": ADE_CONF['user'],
            "password": ADE_CONF['password'],
            "function": "connect",
        },
    )
    session_id = ET.fromstring(response.text).attrib["id"]
    logger.info("📖 Setting project")
    requests.get(
        BASE_URL,
        params={
            "sessionId": session_id,
            "function": "setProject",
            "projectId": ADE_CONF['project_id'],
        },
    )
    logger.info("💾 Fetching data")
    response = requests.get(
        BASE_URL,
        params={
            "sessionId": session_id,
            "function": "getResources",
            "tree": "true",
            "category": "classroom",
            "detail": "0",
        },
    )

    parser = ADE_Parser(S3_client())
    # open('ade.xml','w').write(response.text)
    parser.parse(response.text)



import configparser
import logging
import logging.config
import sys

import schedulesy_qrcode.config as config


def main():
    if len(sys.argv) != 2:
        raise IndexError("Usage: qrcodes <path/to/config>")
    configuration = configparser.ConfigParser()
    configuration.read(sys.argv[1])
    config.ADE_CONF = configuration['ADE']
    config.S3_CONF = configuration['S3']
    config.QR_CONF = configuration['QRCode']
    config.FONT_CONF = configuration['Font']
    config.TEMPLATES_CONF = configuration['Template']
    logging.config.fileConfig(
        sys.argv[1],
        disable_existing_loggers=False,
        defaults={},
    )

    from schedulesy_qrcode.refresh import refresh

    refresh()


if __name__ == '__main__':
    sys.exit(main())



import json

from jinja2 import Environment, FileSystemLoader, PackageLoader, select_autoescape

from schedulesy_qrcode.config import QR_CONF, S3_CONF, TEMPLATES_CONF


def render(data):
    env = Environment(
        loader=FileSystemLoader(TEMPLATES_CONF['path']), autoescape=select_autoescape()
    )
    template = env.get_template("tree.html")

    return template.render(
        data=data,
        s3={
            "endpoint": S3_CONF['endpoint'],
            "bucket": S3_CONF['bucket'],
            "tenant": S3_CONF['tenant'],
        },
        target={"url": QR_CONF['url']},
    )
